@@ -2,17 +2,17 @@
  * This is libcdplay, a Linux-specific CD-ROM playing library. 
  *
  * This code is (C) 1998-2001 David E. Smith <dave@technopagan.org>
- * and released under the GNU LGPL. See `COPYING.LIB' for details.
+ * and released under the GNU GPL. See `COPYING' for details.
  *
- * Significant changes/improvements made by Lorenz Minder <lminder@gmx.net>
- * who's just so incredibly cool I felt I needed to mention him here. :-)
+ * (Old versions were under the LGPL. This is no longer the case.)
  *
  */
 
 #include "libcdplay.h"
 #include "sha.h"
-#include "base64.h"
+#include "mbo.h"
 #include "dcd.h"
+#include "base64.h"
 #include <unistd.h>
 #include <string.h> 
 #include <errno.h>
@@ -24,7 +24,12 @@
 #include <signal.h>
 
 
+#define DISC_MAGIC 42
 static int cdrive;			/* fd for the cd, natch */
+static int disc_present = DISC_MAGIC;	/* ugly hack for cd_present() */
+static char discid[33];
+
+static void int_cd_discid();
 
 /* Plays just the one track specified by trknum. */
 u_char cd_play_track (u_char trknum) {
@@ -32,6 +37,7 @@ u_char cd_play_track (u_char trknum) {
   #ifdef DEBUG
     fprintf (stderr,"Entering cd_play_track (%i)\n", trknum);
   #endif
+  if (!cd_present()) return -1;
   memset (&trkidx, 0, sizeof (struct cdrom_ti)); 
   trkidx.cdti_trk0 = trkidx.cdti_trk1 = trknum;
   if (cd_data_track(trknum)) return -1;
@@ -62,7 +68,7 @@ u_char cd_play_track (u_char trknum) {
     play.cdmsf_sec1=toce1.cdte_addr.msf.second;
     play.cdmsf_frame1=toce1.cdte_addr.msf.frame;
     if(ioctl (cdrive, CDROMPLAYMSF, &play) == -1)
-      fprintf(stderr, "CDROMPLAYMSF failed: %s\n", strerror(errno));
+     fprintf(stderr, "CDROMPLAYMSF failed: %s\n", strerror(errno));
   }
   return trknum;
 } /* cd_play_track */
@@ -73,6 +79,7 @@ u_char cd_play_sequence (u_char trk1, u_char trk2) {
   #ifdef DEBUG
     fprintf (stderr, "Entering cd_play_sequence (%i to %i)\n", trk1, trk2);
   #endif
+  if (!cd_present()) return -1;
   memset (&trkidx, 0, sizeof (struct cdrom_ti));
   trkidx.cdti_trk0 = trk1;
   trkidx.cdti_trk1 = trk2;
@@ -114,6 +121,7 @@ u_char cd_play_disc (u_char trknum) {
   #ifdef DEBUG
     fprintf (stderr,"Entering cd_play_disc (%i)\n", trknum);
   #endif
+  if (!cd_present()) return -1;
   memset (&trkidx, 0, sizeof(struct cdrom_ti));
   trkidx.cdti_trk0 = trknum;
   trkidx.cdti_trk1 = cd_last_track();
@@ -178,6 +186,7 @@ u_char cd_current_track() {
   #ifdef DEBUG
     fprintf (stderr, "Calling cd_current_track()... ");
   #endif
+  if (!cd_present()) return 0;
   memset (&subchnl, '\0', sizeof(struct cdrom_subchnl));
   subchnl.cdsc_format = CDROM_MSF;
   if(ioctl(cdrive, CDROMSUBCHNL, &subchnl) == -1)
@@ -192,6 +201,7 @@ u_char cd_current_track() {
 unsigned long raw_track_length (u_char trknum) {
   long frames = 0;
   struct cdrom_tocentry toce;
+  if (!cd_present()) return 0;
   toce.cdte_format = CDROM_MSF;
   toce.cdte_track = (trknum == cd_last_track()) ? CDROM_LEADOUT : trknum+1;
   if(ioctl (cdrive, CDROMREADTOCENTRY, &toce) == -1)
@@ -205,6 +215,7 @@ unsigned long raw_track_length (u_char trknum) {
 unsigned long cd_track_length_frames (u_char trknum) {
   long frames = raw_track_length(trknum);
   long prevframes = raw_track_length (trknum-1);
+  if (!cd_present()) return 0;
   frames -= prevframes;
   #if DEBUG > 1
     fprintf (stderr, "cd_track_length_frames: track %i is %lu frames\n",
@@ -217,6 +228,7 @@ unsigned long cd_track_length_frames (u_char trknum) {
 int cd_track_length (u_char trknum) {
   long frames = cd_track_length_frames(trknum);
   int seconds = frames / CD_FRAMES;
+  if (!cd_present()) return 0;
   if ((frames % CD_FRAMES) > (CD_FRAMES / 2)) seconds++;
   #ifdef DEBUG
     fprintf (stderr, "cd_track_length: track %i is %i seconds\n",
@@ -230,12 +242,14 @@ unsigned long cd_disc_length_frames (void) {
   #if DEBUG > 1
     fprintf (stderr, "cd_disc_length_frames: %lu\n", frames);
   #endif
+  if (!cd_present()) return 0;
   return frames;
 }
 
 int cd_disc_length (void) {
   long frames = cd_disc_length_frames();
   int seconds = frames / CD_FRAMES;
+  if (!cd_present()) return 0;
   if ((frames % CD_FRAMES) > (CD_FRAMES / 2)) seconds++;
   #ifdef DEBUG
     fprintf (stderr, "cd_disc_length: %i\n", seconds);
@@ -261,6 +275,24 @@ int cd_hw_status (void) {
    Unless you know what you're doing, you probably don't want to use this.
 */
 
+/* this was Othmar's, but I sorta reversed it. */
+int cd_present(void) {
+  int status;
+  if (disc_present != DISC_MAGIC) status = disc_present;
+    else {
+      status = ioctl(cdrive, CDROM_DRIVE_STATUS);
+      disc_present = status;
+    }
+  #ifdef DEBUG
+    // fprintf (stderr, "cd_present(), status is %i\n", status);
+  #endif
+  if ((status == CDS_DISC_OK) || (status == CDS_NO_INFO)) return TRUE;
+  /* returning TRUE if the drive doesn't support the status ioctl is,
+     perhaps, risky, but one can only wonder... */
+  /* other status codes: no disc, tray open, drive not ready */
+  return FALSE;
+}
+
 /* toggle pause state */
 void cd_pause (void) {
   int pause = cd_hw_status();
@@ -276,7 +308,11 @@ void cd_pause (void) {
       if(ioctl (cdrive, CDROMPAUSE) == -1)
 	fprintf(stderr, "CDROMPAUSE failed: %s\n", strerror(errno));
       break;
-    default: break; /* bah who needs error handling */
+    default: cd_play_disc(cd_first_track()); break;
+    /* the above was a really cool idea from Ronald Tol             */
+    /* Any state other than CDROM_AUDIO_PLAY implies that we're not */
+    /* playing right now, so what the hell? :) linux/cdrom.h lists  */
+    /* all the possible states, and this seems to make sense...     */
   }
 }
 
@@ -312,6 +348,7 @@ u_char cd_first_track (void) {
   #ifdef DEBUG
     fprintf (stderr, "Calling cd_first_track()... ");
   #endif
+  if (!cd_present()) return 0;
   if(ioctl (cdrive, CDROMREADTOCHDR, &tochdr) == -1)
     fprintf(stderr, "CDROMREADTOCHDR failed: %s\n", strerror(errno));
   #ifdef DEBUG
@@ -325,6 +362,7 @@ u_char cd_last_track (void) {
   #ifdef DEBUG
     fprintf (stderr, "Calling cd_last_track()... ");
   #endif
+  if (!cd_present()) return 0;
   if(ioctl (cdrive, CDROMREADTOCHDR, &tochdr) == -1)
     fprintf(stderr, "CDROMREADTOCHDR failed: %s\n", strerror(errno));
   #ifdef DEBUG
@@ -336,11 +374,14 @@ u_char cd_last_track (void) {
 int cd_data_track (u_char trknum) {
   struct cdrom_tocentry toce;
   int i;
+  if (!cd_present()) return -1;
   memset (&toce, '\0', sizeof(struct cdrom_tocentry));
   toce.cdte_format = CDROM_MSF;
   toce.cdte_track  = trknum;
-  if(ioctl (cdrive, CDROMREADTOCENTRY, &toce) == -1)
-    fprintf(stderr, "CDROMREADTOCENTRY[4] failed: %s\n", strerror(errno));
+  ioctl(cdrive, CDROMREADTOCENTRY, &toce);
+  /* if(ioctl (cdrive, CDROMREADTOCENTRY, &toce) == -1) */
+    /* fprintf(stderr, "CDROMREADTOCENTRY[4] failed: %s\n", strerror(errno)); */
+    /* apparently this fails randomly on some drives for no good reason. */
   i = (toce.cdte_ctrl & CDROM_DATA_TRACK ? TRUE : FALSE);
   #ifdef DEBUG
     fprintf (stderr, "toce.cdte_ctrl is %i\n", toce.cdte_ctrl);
@@ -352,9 +393,16 @@ int cd_data_track (u_char trknum) {
 /* Hey, wow, it's the mandatory `initialize the drive' function. */
 int cd_init_player (char *device) {
   #ifdef DEBUG
-    fprintf (stderr, "Calling cd_init_player(%s). Debugging ON.\n", device);
-  #endif 
-  if (!(cdrive = open(device, O_RDONLY | O_NONBLOCK))) return -1;
+    fprintf(stderr, "Calling cd_init_player(%s). Debugging ON.\n", device);
+  #endif
+
+  cdrive = open(device, O_RDONLY | O_NONBLOCK);
+  if (!cdrive) return -1;
+
+  if (!cd_present()) return -1;
+
+  int_cd_discid();
+  mbo_init(device);  
   return cdrive;
 }
 
@@ -363,20 +411,31 @@ int cd_fd (void) {
   return cdrive;
 }
 
-/* the discid and sub-id functions were, um, liberated from the CD Index
- * reference implementation. More or less, anyway. 
+/* Okay, this is just f'n silly. musicbrainz has a "Get CD Index"
+ * function, but it won't work unless you've already retrieved
+ * the information from the network. So I have to keep all the
+ * "Get CD Index ID" code here.
  */
 
-char *cd_discid (void) {
+char *cd_discid(void) {
+  return discid;
+}
+
+static void int_cd_discid (void) {
   SHA_INFO sha;
-  unsigned char digest[20], *base64;
-  char *discid = malloc(33);
+  unsigned char digest[20];
   unsigned long size;
   char temp[9];
   int i, lba;
   int trackz = cd_last_track();
   struct cdrom_tocentry toce;
 
+  if (!cd_present()) return;;
+
+  #if DEBUG > 1
+    fprintf (stderr, "Entering cd_discid()\n");
+  #endif
+
   memset (discid, '\0', sizeof(discid));
   memset (&toce, '\0', sizeof(toce));
 
@@ -407,49 +466,17 @@ char *cd_discid (void) {
     sha_update(&sha, (unsigned char *)temp, strlen(temp));
   }
   sha_final(digest, &sha);
-  base64 = rfc822_binary(digest, 20, &size);
-  memcpy(discid, base64, size);
-  free(base64); 
+  #if DEBUG > 1
+    fprintf (stderr, "SHA digest finalized.\n");
+  #endif
+  bin_to_base64(discid, digest, 20, size);
   #ifdef DEBUG
     fprintf (stderr, "cd_discid returning %s\n", discid);
   #endif
-  return discid;
 }
 
-char *cd_subid (void) {
-  char *sub_id = malloc(1024);
-  int i, lba;
-  int tracka = cd_first_track();
-  int trackz = cd_last_track();
-  struct cdrom_tocentry toce;
-
-  memset (&toce, '\0', sizeof(toce));
-  memset (sub_id, '\0', sizeof(sub_id));
-
-  sprintf (sub_id,
-          "http://www.cdindex.org/cgi-bin/cdi/submit.pl?id=%s&tracks=%d&toc=",
-          cd_discid(), (trackz-tracka)+1 );
-
-  toce.cdte_track = CDROM_LEADOUT;
-  toce.cdte_format = CDROM_LBA;
-  if(ioctl(cdrive, CDROMREADTOCENTRY, &toce) == -1)
-    fprintf(stderr, "CDROMREADTOCENTRY[7] failed: %s\n", strerror(errno));
-  lba = toce.cdte_addr.lba + 150;
-  sprintf (sub_id + strlen(sub_id), "%d+%d+%d", tracka, trackz, lba);
-
-  for (i=1; i < 100; i++) {
-    if (i <= trackz) {
-      toce.cdte_format = CDROM_LBA;
-      toce.cdte_track = i;
-      if(ioctl (cdrive, CDROMREADTOCENTRY, &toce) == -1)
-	fprintf(stderr, "CDROMREADTOCENTRY[8] failed: %s\n", strerror(errno));
-      lba = toce.cdte_addr.lba + 150;
-      sprintf(sub_id + strlen(sub_id), "+%d", lba);
-    }
-  }
-  #ifdef DEBUG
-    fprintf (stderr, "cd_subid returning %s\n", sub_id);
-  #endif
-  return sub_id;
-}
+/* there used to be more stuff here, but it's been more-or-less
+   subsumed by the musicbrainz crapulence. */
 
+/* char *cd_discid(void); */
+/* char *cd_subid(void);  */
